import numpy as np
import pandas as pd
# For visualizations
import matplotlib.pyplot as plt
# For regular expressions
import re
# For handling string
import string
# For performing mathematical operations
import math
import spacy
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction import text

from wordcloud import WordCloud
from textwrap import wrap

def create_word_cloud(import_file_path,stop_words):
    """
    The function will create Word Cloud using Document Term Matrix.
    
    Parameters:
        import_file_path (string): File Path to export the data.  
                                   There should be two columns in the file: "use_case", "all_text".
                                   File name should be .csv extension.
        stop_words (list/string): Enter the list of stop words, which should be avoided while creating word cloud. Example: "mother, mom, mum"
    Example:
        create_word_cloud("D:\\\\Analysis\\\\Occassion Analysis\\\\test\\\\occasion.csv","mother, mom, mum, mummy, grandmother, grand mother, grand ma, grandma, step mom, stepmom") --Stop words for Mother''s Day
    """
    # Importing dataset
    df = pd.read_csv(
        import_file_path,
        low_memory=False,
    )
    df = df[["use_case", "all_text"]]

    print("File Read!")

    # remove nulls
    df = df.dropna(how="any", axis=0)
    df.dropna(inplace=True)
    # convert ASCII to Unicode Emojis
    def deEmojify(inputString):
        return inputString.encode("ascii", "ignore").decode("ascii")


    df["all_text"] = df["all_text"].apply(lambda x: deEmojify(x))
    # remove commas
    df["all_text"] = df["all_text"].apply(lambda x: x.split(",,,")[0])
    # lower case
    df["cleaned"] = df["all_text"].apply(lambda x: x.lower())
    # remove digits
    df["cleaned"] = df["cleaned"].apply(lambda x: re.sub("\w*\d\w*", "", x))
    # remove punctuation
    df["cleaned"] = df["cleaned"].apply(
        lambda x: re.sub("[%s]" % re.escape(string.punctuation), "", x)
    )
    # Removing extra spaces
    df["cleaned"] = df["cleaned"].apply(lambda x: re.sub(" +", " ", x))
    df["cleaned"] = df["cleaned"].str.encode("ascii", "ignore").str.decode("ascii")
    # Remove unwanted words like I, will, you etc.
    nlp = spacy.load("en_core_web_sm", disable=["parser", "ner"])
    # Normalizing the data like Played, Playing, Play should fall under Play
    df["lemmatized"] = df["cleaned"].apply(
        lambda x: " ".join(
            [token.lemma_ for token in list(nlp(x)) if (token.is_stop == False)]
        )
    )

    # Group the data by Use Case
    df_grouped = (
        df[["use_case", "lemmatized"]].groupby(by="use_case").agg(lambda x: " ".join(x))
    )

    print("Data Cleaning and Grouping Done!")

    # Create Document Term Matrix
    stop_words = stop_words + (",xx,xxxx,xxx,day,dear,hi,hello")
    stop_words_list = list(stop_words.split(","))
    stop_words_list = [x.lstrip(" ").lower() for x in stop_words_list]
    print(stop_words_list)
    cv = CountVectorizer(analyzer="word", stop_words=stop_words_list, min_df=0.10)
    data = cv.fit_transform(df_grouped["lemmatized"])
    df_dtm = pd.DataFrame(data.toarray(), columns=cv.get_feature_names_out())
    df_dtm.index = df_grouped.index

    print("Document Term Matrix created!")
    
    ## Function for generating word clouds
    def generate_wordcloud(data, title):
        wc = WordCloud(
            width=400, height=330, max_words=150, colormap="Dark2"
        ).generate_from_frequencies(data)
        plt.figure(figsize=(10, 8))
        plt.imshow(wc, interpolation="bilinear")
        plt.axis("off")
        plt.title("\n".join(wrap(title, 60)), fontsize=13)
        plt.show()


    # Transposing document term matrix
    df_dtm = df_dtm.transpose()


    # Plotting word cloud for each product
    for index, product in enumerate(df_dtm.columns):
        generate_wordcloud(df_dtm[product].sort_values(ascending=False), product)



import psycopg2
from psycopg2 import sql
import os
import csv
import io
from dotenv import load_dotenv

def export_data_to_csv(credential_file_path, export_file_path):
    """
    The function will export data based on SQL query into csv file.
    For data loading, create a file named: "credentials.env" which has database credential details in following format:
        DATABASE = 'database_name'
        USER = 'user_name'
        PASSWORD = 'password'
        HOST = 'host'
        PORT = 'port_number'
  
    Parameters:
        credential_file_path (string): File Path for Credential File with file name.
        export_file_path (string): File Path to export the data with file name.
    Example:
        export_data_to_csv("D:\\\\Analysis\\\\Occassion Analysis\\\\credentials.env","C:\\\\Analysis\\\\Occassion Analysis\\\\test\\\\occasion.csv")
    """
    load_dotenv(credential_file_path)

    #this section is for converting text/symbolic emoticons
    def  emoji_converter(message):
            words = message.split(" ")
            emojis = {
            ":)" : "😀",
            ":(" : "😞",
            ":-)" : "😀",
            ":-(" : "😞",
            ":D":"😄",
            ":-D":"😄",
            ":*":"😘",
            ":-*":"😘",
            ":x":"😘",
            ":P":"😛",
            ":-P":"😛",
            ":p":"😛",
            ":-p":"😛"
            }
            outcome = " "
            for word in words:
                outcome += emojis.get(word, word) + " "
            return outcome


    dbname = os.getenv('DATABASE')
    host = os.getenv('HOST')
    port = os.getenv('PORT')
    user = os.getenv('USER')
    password = os.getenv('PASSWORD')

    try:
    #establishing the connection
        conn = psycopg2.connect(
            database = dbname, user = user, password = password, host = host, port = port
        )
        print("Connection successful")
    except psycopg2.DatabaseError as e:
        # Confirm unsuccessful connection and stop program execution.
            print("Database connection unsuccessful.",e)
            quit()

    #Creating a cursor object using the cursor() method
    cursor = conn.cursor()

    t_path_n_file = export_file_path 
    occasion_input = input("Enter occasion name (if input is Father's Day or Mother's Day then use two inverted comma ''s Day): ")
    if occasion_input == '':
        print('Null Input,quiting')
        quit()
    else:
        sql_query = ("SELECT tier_2_use_case as use_case, COALESCE(all_message_text(messages),'') as all_text_new from orders.ordered_products op "
                    f"JOIN (SELECT tier_2_use_case, ordered_product_id from looker.tiered_attributes_new where UPPER(tier_2_use_case) = UPPER('{occasion_input}') GROUP BY 1,2)uc ON op.ordered_product_id = uc.ordered_product_id "
                    "GROUP BY 1,2")
        print(sql_query)
        cursor.execute(sql_query)

        export_list = cursor.fetchall()

        try:
            with  io.open(t_path_n_file, "w",encoding = 'utf-8-sig') as file:
                writer = csv.writer(file, delimiter=',')
                writer.writerow(['use_case','all_text'])
                #adding set of code to convert text emoji into unicode emoji
                for row in export_list:
                    row_list = list(row)
                    text_to_encode = emoji_converter(row[1])
                    row_list.append(text_to_encode)
                    row_1 = list()
                    row_1.append(row_list[0])
                    row_1.append(row_list[2])
                    row_final = tuple(row_1)
                    writer.writerow(row_final)
            print("Query executed successfully")
        except psycopg2.databaseerror as e:
            print("Error is: ",e)
            quit()
    cursor.close()
    conn.close()



import pandas as pd
# For creating frequency chart
import plotly.graph_objects as go
import plotly.io as pio
import plotly.express as px
import plotly
# For getting emoji count
import advertools as adv


def create_emoji_chart(import_file_path):
    """
    The function will create Emoji Frequency Chart.

    Parameters:
        import_file_path (string): File Path to export the data.
                                   There should be two columns in the file: "use_case", "all_text".
                                   File name should be .csv extension.
    Example:
        create_emoji_chart("D:\\\\Analysis\\\\Occassion Analysis\\\\test\\\\occasion.csv")
    """
    df = pd.read_csv(import_file_path, low_memory=False, encoding="utf-8-sig")
    df = df[["use_case", "all_text"]]
    print("File Read!")
    text = df["all_text"]

    ##Getting Emoji Summary
    emoji_summary = adv.extract_emoji(text)
    emoji_counts = emoji_summary["top_emoji"]

    emoji_counter = pd.DataFrame(emoji_counts, columns=["Emoji", "Count"])
    emoji_counter.set_index("Emoji")
    max_value = emoji_counter._get_value(0, "Count")

    fig = go.Figure()

    fig.add_trace(
        go.Scatter(
            y=emoji_counter["Emoji"],
            x=emoji_counter["Count"],
            name="Emoji Counts",
            marker_color="white",
            orientation="h",
            text=emoji_counter["Emoji"],
            textposition="top center",
            mode="markers+text",
            textfont=dict(size=25),
        )
    )

    fig.update_yaxes(visible=False, range=[50, 0])
    fig.update_xaxes(
        title="Number of Times Used Range 50-500", range=[50, 500]
    )  # 1000 instead of using max_value

    fig.update_layout(
        template="simple_white",
        # height=len(emoji_counter)*0.5, width = 1500
    )
    fig.show()
